/*
 * (C) Copyright 2002
 * Hyperion Entertainment, Hans-JoergF@hyperion-entertainment.com
 *
 * modified 2008-2015 by
 * Max Tretene, ACube Systems Srl. mtretene@acube-systems.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <pci.h>
#include "Sam440ep_flex.h"
#include "../common/vesa.h"

DECLARE_GLOBAL_DATA_PTR;

#undef DEBUG

#ifdef  DEBUG
#define PRINTF(fmt,args...)     printf (fmt ,##args)
#else
#define PRINTF(fmt,args...)
#endif

#define MAX_GFX_CARDS 3

extern int execute_bios(pci_dev_t gr_dev, void *);

static short classes[] =
{
	PCI_CLASS_DISPLAY_VGA,
	PCI_CLASS_DISPLAY_XGA,
	PCI_CLASS_DISPLAY_3D,
	PCI_CLASS_DISPLAY_OTHER,
	~0
};

int dvi_present = 0;
int active_card = 0;

//u32 *fb_base_phys;
u32 mmio_base_phys;

struct FrameBufferInfo *tmp_fbi[MAX_GFX_CARDS];
extern struct FrameBufferInfo *fbi;

/*
 * For a given bus number, find the bridge on this hose that provides this
 * bus number. The function scans for bridges and peeks config space offset
 * 0x19 (PCI_SECONDARY_BUS).
 */
pci_dev_t pci_find_bridge_for_bus(struct pci_controller *hose, int busnr)
{
	pci_dev_t dev;
	int bus;
	unsigned int found_multi=0;
	unsigned char header_type;
	unsigned short vendor;
	unsigned char secondary_bus;

	if (hose == NULL) return PCI_ANY_ID;

	if (busnr < hose->first_busno || busnr > hose->last_busno) return PCI_ANY_ID; /* Not in range */

	/*
	 * The bridge must be on a lower bus number
	 */
	for (bus = hose->first_busno; bus < busnr; bus++)
	{
		for (dev =  PCI_BDF(bus,0,0);
		     dev <  PCI_BDF(bus,PCI_MAX_PCI_DEVICES-1,PCI_MAX_PCI_FUNCTIONS-1);
		     dev += PCI_BDF(0,0,1))
		{
		    if ( dev == PCI_BDF(hose->first_busno,0,0) )
			continue;

		    if (PCI_FUNC(dev) && !found_multi)
			continue;

		    pci_hose_read_config_byte(hose, dev, PCI_HEADER_TYPE, &header_type);

		    pci_hose_read_config_word(hose, dev, PCI_VENDOR_ID, &vendor);

		    if (vendor != 0xffff && vendor != 0x0000)
		    {
				if (!PCI_FUNC(dev))
				    found_multi = header_type & 0x80;
				if (header_type == 1) /* Bridge device header */
				{
				    pci_hose_read_config_byte(hose, dev, PCI_SECONDARY_BUS, &secondary_bus);
				    if ((int)secondary_bus == busnr) return dev;
				}
		    }
		}
	}
	return PCI_ANY_ID;
}

pci_dev_t pci_hose_find_class(struct pci_controller *hose, int bus, short find_class, int index)
{
	unsigned int sub_bus, found_multi=0;
	unsigned short vendor, class;
	unsigned char header_type;
	pci_dev_t dev;
	u8 c1, c2;

	sub_bus = bus;

	for (dev =  PCI_BDF(bus,0,0);
	 dev <  PCI_BDF(bus,PCI_MAX_PCI_DEVICES-1,PCI_MAX_PCI_FUNCTIONS-1);
	 dev += PCI_BDF(0,0,1))
	{
		if ( dev == PCI_BDF(hose->first_busno,0,0) )
		    continue;

		if (PCI_FUNC(dev) && !found_multi)
		    continue;

		pci_hose_read_config_byte(hose, dev, PCI_HEADER_TYPE, &header_type);

		pci_hose_read_config_word(hose, dev, PCI_VENDOR_ID, &vendor);

		if (vendor != 0xffff && vendor != 0x0000)
		{
		    if (!PCI_FUNC(dev))
			found_multi = header_type & 0x80;
		    pci_hose_read_config_byte(hose, dev, 0x0B, &c1);
		    pci_hose_read_config_byte(hose, dev, 0x0A, &c2);
		    class = c1<<8 | c2;

		    if (class == find_class)
		    {
				if (index == 0)
				    return dev;
				else index--;
		    }
		}
	}

	return ~0;
}

int init_vga (struct pci_controller *hose)
{
	char *s;
	int ii,jj, no_irq = 0;
	int busnr = 0;
	int classnr = 0;
	pci_dev_t dev = ~0;
	pci_dev_t bridge = ~0;

	pci_dev_t ddev_a[MAX_GFX_CARDS];
	pci_dev_t bridgedev[MAX_GFX_CARDS];

	short devCmd[MAX_GFX_CARDS];

	int nVideoCardActive = -1;
	int nVideoCard = 0;

	unsigned char agp_control_0;
	unsigned short cmd;
	u8 subcode;

	for (ii=0; ii < MAX_GFX_CARDS; ii++)
	{
	    ddev_a[ii] =  ~0;
	    bridgedev[ii] = ~0;
	}

	puts("VGA:   ");

	PRINTF("Trying to initialize x86 VGA Card(s)\n");

	// Phase 1 ------------------------------------------------------

	while (classes[classnr] != ~0)
	{
		for(busnr = 0; busnr <= hose->last_busno; busnr++)
		{
			PRINTF("Searching for class 0x%x on bus %d\n", classes[classnr],
				 busnr);

			/* Find the first of this class on this bus */

			dev = pci_hose_find_class(hose, busnr, classes[classnr], 0);

			if (dev != ~0)
			{
				pci_hose_read_config_byte(hose, dev, PCI_CLASS_SUB_CODE, &subcode);

				if (subcode <= 0x7F)
				{
		   			PRINTF("Found VGA Card at %02x:%02x:%02x\n", PCI_BUS(dev), PCI_DEV(dev), PCI_FUNC(dev));
					ddev_a[nVideoCard] = dev;

					if (PCI_BUS(dev) != 0)
					{
						bridge = pci_find_bridge_for_bus(hose, PCI_BUS(dev));
						PRINTF("Behind bridge (%d) at %02x:%02x:%02x\n", bridge,
							PCI_BUS(bridge), PCI_DEV(bridge), PCI_FUNC(bridge));
						bridgedev[nVideoCard] = bridge;
					}

					nVideoCard++;
				}
			}
		}

		classnr ++;
	}

	if(nVideoCard == 0)
	{
		puts("NO CARDS\n");
		return -1;
	}

	PRINTF("Number of VGA cards detected: %d\n", nVideoCard);

	/*
	 * If we get here we have found the first graphics card.
	 * If the bus number is not 0, then it is probably behind a bridge, and the
	 * bridge needs to be told to forward VGA access.
	 */

	// Phase 2 ------------------------------------------------------

	PRINTF("Shutting down I/O on all graphics cards\n");
	for (jj=0; jj < MAX_GFX_CARDS; jj++)
	{
		if (ddev_a[jj] != ~0)
		{
			PRINTF("Shutting down graphics card at %x.%x.%x\n",
				PCI_BUS(ddev_a[jj]), PCI_DEV(ddev_a[jj]), PCI_FUNC(ddev_a[jj]));

			// Graphics card...
			pci_hose_read_config_word(hose, ddev_a[jj], PCI_COMMAND, &cmd);

			devCmd[jj] = cmd;
			cmd &= ~(PCI_COMMAND_IO|PCI_COMMAND_MEMORY);

			pci_hose_write_config_word(hose, ddev_a[jj], PCI_COMMAND, cmd);
			PRINTF("    CMD register now %X\n", cmd);
		}
	}

	// Phase 3 ------------------------------------------------------

	for (ii = 0; ii < nVideoCard; ii++)
	{
	 	dev = ddev_a[ii];

	 	PRINTF("Ciclo: %d BUS: %d\n",ii,PCI_BUS(dev));

	    if (PCI_BUS(dev) != 0)
		{
			PRINTF("Behind bridge, looking for bridge\n");
			bridge = bridgedev[ii];
			PRINTF("bridge %d\n",bridge);
			if (bridge != PCI_ANY_ID)
			{
		    	PRINTF("Got the bridge at %02x:%02x:%02x\n",
				   PCI_BUS(bridge), PCI_DEV(bridge), PCI_FUNC(bridge));

			    pci_hose_read_config_byte(hose, bridge, 0x3E, &agp_control_0);
	    		agp_control_0 |= 0x18;
			    pci_hose_write_config_byte(hose, bridge, 0x3E, agp_control_0);

			    if (PCI_BUS(dev) == 2)
			    {
			    	PRINTF("BRIDGE after BRIDGE !!!\n");
			    	bridge = pci_find_bridge_for_bus(hose, PCI_BUS(bridge));
			    	if (bridge != PCI_ANY_ID)
					{
				    	PRINTF("Got the bridge at %02x:%02x:%02x\n",
						   PCI_BUS(bridge), PCI_DEV(bridge), PCI_FUNC(bridge));

					    pci_hose_read_config_byte(hose, bridge, 0x3E, &agp_control_0);
			    		agp_control_0 |= 0x18;
					    pci_hose_write_config_byte(hose, bridge, 0x3E, agp_control_0);

					}
				}

			    PRINTF("Configured for VGA forwarding\n");
			}
	    }
	}

	/*
	 * Now try to run the bios
	 */

	dvi_present = 0;

	s = getenv("video_activate");
	if (!s) s = "internal"; // defualt value

	char *aa = getenv("ext_gfx_irq");
	if (strcmp(aa,"disabled") == 0)
		no_irq = 1;

	for (ii = 0; ii < nVideoCard; ii++)
	{
	 	dev = ddev_a[ii];

		active_card = 1;

		if ((ii == 0) && (nVideoCard > 1) && (strcmp(s, "pci") == 0))
	    {
	    	active_card = 0;
		}

	    if ((ii > 0) && (strcmp(s, "internal") == 0))
	    {
	    	active_card = 0;
	    }

		PRINTF("\nTrying to run bios on card #%d now\n", ii);

		// enable current card
		cmd = devCmd[ii];
		cmd |= PCI_COMMAND_IO | PCI_COMMAND_MEMORY;
		pci_hose_write_config_word(hose, dev, PCI_COMMAND, cmd);

		if (execute_bios(dev, (void *)gd->relocaddr))
		{
			printf("%d ",ii+1);

			tmp_fbi[ii] = set_vesa_mode(257);

			++nVideoCardActive;
		}

		// disable current card
		cmd = devCmd[ii];
		cmd &= ~(PCI_COMMAND_IO|PCI_COMMAND_MEMORY);
		pci_hose_write_config_word(hose, dev, PCI_COMMAND, cmd);
	}

	for (ii = 0; ii < nVideoCard; ii++)
	{
	 	dev = ddev_a[ii];

		active_card = 1;

		if ((ii == 0) && (nVideoCard > 1) && (strcmp(s, "pci") == 0))
	    {
	    	active_card = 0;
		}

	    if ((ii > 0) && (strcmp(s, "internal") == 0))
	    {
	    	active_card = 0;
	    }

		// enable preferred card
		if (active_card)
		{
			PRINTF("Re-enabling %x.%x.%x\n",
				PCI_BUS(dev), PCI_DEV(dev), PCI_FUNC(dev));
			cmd = devCmd[ii];
			cmd |= PCI_COMMAND_IO | PCI_COMMAND_MEMORY;
			pci_hose_write_config_word(hose, dev, PCI_COMMAND, cmd);

			pci_read_config_dword(dev, PCI_BASE_ADDRESS_2, &mmio_base_phys);
			mmio_base_phys &= ~0x04;
			PRINTF("mmio_base_phys %8x\n",mmio_base_phys);

			fbi = tmp_fbi[ii];
		}

		if ((ii > 0) && (no_irq))
		{
			pci_hose_write_config_word(hose, dev, PCI_INTERRUPT_LINE, 0xff);
		}
	}

	puts("\n");

	// assign the PMM2 registers to the last card
	// the onboard one is alway the first since it's ont the primary PCI bus

	u32 addr, size;
	dev = ddev_a[nVideoCardActive];
	pci_read_config_dword(dev, PCI_BASE_ADDRESS_0, &addr);
	addr = addr & PCI_BASE_ADDRESS_MEM_MASK;
	size = get_bar_size(dev, PCI_BASE_ADDRESS_0);
	PRINTF("PMM2 %08x %08x (%d)\n",addr, size, nVideoCardActive);

	out32r(PCIX0_PMM2MA, 0);
	out32r(PCIX0_PMM2LA, addr);
	out32r(PCIX0_PMM2PCILA, addr);
	out32r(PCIX0_PMM2MA, ~(size-1) | 3);

	return nVideoCardActive;
}
