
/*
 * (C) Copyright 2008-2009
 * Max Tretene, ACube Systems Srl. mtretene@acube-systems.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/************************************************************************
 * sam440ep.h - configuration for Sam440ep board
 ***********************************************************************/
#ifndef __CONFIG_H
#define __CONFIG_H

/*-----------------------------------------------------------------------
 * High Level Configuration Options
 *----------------------------------------------------------------------*/

#define CONFIG_SAM440EP		1			/* Board is Sam440EP            */
#define CONFIG_440EP		1			/* Specific PPC440EP support    */
#define CONFIG_4xx			1			/* ... PPC4xx family	        */
#define CONFIG_SYS_CLK_FREQ	66666666    /* external freq to pll	*/

#define CONFIG_LAST_STAGE_INIT
#define CONFIG_BOARD_EARLY_INIT_F 1     /* Call board_early_init_f	*/
#define CONFIG_MISC_INIT_R	1			/* call misc_init_r()		*/

#define CONFIG_BOARD_RESET	1			/* call board_reset()		*/
#define CONFIG_VERY_BIG_RAM 1


/*-----------------------------------------------------------------------
 * Base addresses -- Note these are effective addresses where the
 * actual resources get mapped (not physical addresses)
 *----------------------------------------------------------------------*/
#define CFG_MONITOR_LEN			(512 * 1024)		/* Reserve 512 kB for Monitor	*/
#define CFG_MALLOC_LEN			(32 * 1024 * 1024)	/* Reserve 8 Mb for malloc() was 256 kB */
#define CFG_MONITOR_BASE		(-CFG_MONITOR_LEN)
#define CFG_SDRAM_BASE	        0x00000000	    /* _must_ be 0	*/
#define CFG_FLASH_BASE			0xfff80000	    /* start of FLASH	*/
#define CFG_FLASH_MAX_SIZE		0x00080000
#define CFG_PCI_MEMBASE	        0xa0000000	    /* mapped pci memory*/
#define CFG_PCI_MEMBASE1        CFG_PCI_MEMBASE  + 0x10000000
#define CFG_PCI_MEMBASE2        CFG_PCI_MEMBASE1 + 0x10000000
#define CFG_PCI_MEMBASE3        CFG_PCI_MEMBASE2 + 0x10000000

/*Don't change either of these*/
#define CFG_PERIPHERAL_BASE		0xef600000	    /* internal peripherals*/
#define CFG_PCI_BASE			0xe0000000	    /* internal PCI regs*/
/*Don't change either of these*/
#define CFG_USB_DEVICE          0x50000000
#define CFG_NVRAM_BASE_ADDR     0x80000000
#define CFG_BOOT_BASE_ADDR      0xf0000000

/*-----------------------------------------------------------------------
 * Initial RAM & stack pointer (placed in SDRAM)
 *----------------------------------------------------------------------*/
#define CFG_INIT_RAM_DCACHE		1		/* d-cache as init ram	*/
#define CFG_INIT_RAM_ADDR		0x70000000		/* DCache       */
#define CFG_INIT_RAM_END		(8 << 10)
#define CFG_GBL_DATA_SIZE		256			/* num bytes initial data*/
#define CFG_GBL_DATA_OFFSET		(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET		CFG_GBL_DATA_OFFSET
#undef CFG_DRAM_TEST			/* test only */

/*-----------------------------------------------------------------------
 * Serial Port
 *----------------------------------------------------------------------*/
#define CONFIG_BAUDRATE			115200
/*define this if you want console on UART1*/
#undef CONFIG_UART1_CONSOLE
#define CFG_CONSOLE_IS_IN_ENV 1

#define CFG_BAUDRATE_TABLE  \
    {300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 307200}

/*-----------------------------------------------------------------------
 * Environment
 *----------------------------------------------------------------------*/
/*
 * Define here the location of the environment variables (FLASH or EEPROM).
 * Note: DENX encourages to use redundant environment in FLASH.
 */
#if 0
#define CFG_ENV_IS_IN_FLASH     1	/* use FLASH for environment vars	*/
#else
#define CFG_ENV_IS_IN_EEPROM	1	/* use EEPROM for environment vars	*/
#endif

/* Define to allow the user to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE	1

/*-----------------------------------------------------------------------
 * FLASH related
 *----------------------------------------------------------------------*/
#define CFG_NO_FLASH 1

#define CFG_MAX_FLASH_BANKS		1		/* max number of memory banks		*/
#define CFG_MAX_FLASH_SECT		11		/* max number of sectors on one chip	*/

#define CFG_FLASH_ERASE_TOUT	150000	/* Timeout for Flash Erase (in ms)	*/
#define CFG_FLASH_WRITE_TOUT	600		/* Timeout for Flash Write (in ms)	*/

#define CFG_FLASH_EMPTY_INFO			/* print 'E' for empty sector on flinfo */

#ifdef CFG_ENV_IS_IN_FLASH
#define CFG_ENV_SECT_SIZE		0x20000	/* size of one complete sector	*/
#define CFG_ENV_ADDR			(CFG_MONITOR_BASE-CFG_ENV_SECT_SIZE)
#define	CFG_ENV_SIZE			0x2000	/* Total Size of Environment Sector	*/

/* Address and size of Redundant Environment Sector	*/
#define CFG_ENV_ADDR_REDUND	(CFG_ENV_ADDR-CFG_ENV_SECT_SIZE)
#define CFG_ENV_SIZE_REDUND	(CFG_ENV_SIZE)
#endif /* CFG_ENV_IS_IN_FLASH */

/*-----------------------------------------------------------------------
 * DDR SDRAM
 *----------------------------------------------------------------------*/
//#define CONFIG_SPD_EEPROM				/* Use SPD EEPROM for setup             */
#undef CONFIG_SAM_SPD_EEPROM
#undef CONFIG_DDR_ECC					/* don't use ECC			*/
#define CFG_SIMULATE_SPD_EEPROM	0xff	/* simulate spd eeprom on this address	*/
#define SPD_EEPROM_ADDRESS { CFG_SIMULATE_SPD_EEPROM, CFG_SIMULATE_SPD_EEPROM, 0x50, 0x50 }

/*-----------------------------------------------------------------------
 * I2C
 *----------------------------------------------------------------------*/
#define CONFIG_HARD_I2C			1		/* I2C with hardware support	*/
#undef	CONFIG_SOFT_I2C					/* I2C bit-banged		*/
#define CFG_I2C_SPEED			400000	/* I2C speed and slave address	*/
#define CFG_I2C_SLAVE			0x7F

#define CFG_I2C_MULTI_EEPROMS			0
#define CFG_I2C_EEPROM_ADDR				(0xa8>>1)
#define CFG_I2C_EEPROM_ADDR_LEN 		1
#define CFG_EEPROM_PAGE_WRITE_ENABLE
#define CFG_EEPROM_PAGE_WRITE_BITS 		3
#define CFG_EEPROM_PAGE_WRITE_DELAY_MS	10

#ifdef CFG_ENV_IS_IN_EEPROM
#define CFG_ENV_SIZE					0x400	/* Size of Environment vars */
#define CFG_ENV_OFFSET					0x0
#endif /* CFG_ENV_IS_IN_EEPROM */

#undef	CONFIG_BOOTARGS

//#define CONFIG_BOOTDELAY		3	/* autoboot after 5 seconds	*/
#define CONFIG_LOADS_ECHO		1	/* echo on for serial download	*/
#define CFG_LOADS_BAUD_CHANGE	1	/* allow baudrate change	*/

#define CONFIG_MII			1		/* MII PHY management		*/
#define CONFIG_PHY_ADDR		24		/* PHY address, See schematics	*/
#define CONFIG_NET_MULTI	1		/* required for netconsole      */
#define CONFIG_HAS_ETH1		1		/* add support for "eth1addr"	*/
#define CONFIG_PHY1_ADDR	25

#define CFG_RX_ETH_BUFFER	4	  /* Number of ethernet rx buffers & descriptors */

/* Partitions */
#define CONFIG_DOS_PARTITION
#define CONFIG_ISO_PARTITION
#define CONFIG_AMIGA_PARTITION

/* OHCI USB */
/* PCI */
#define CONFIG_USB_OHCI_NEW			1
#define CONFIG_PCI_OHCI				1
#define CONFIG_USB_STORAGE			1
#define CONFIG_USB_KEYBOARD			1
#define CFG_OHCI_SWAP_REG_ACCESS	1
#define CFG_USB_OHCI_MAX_ROOT_PORTS	2
#define CFG_USB_OHCI_SLOT_NAME		"ohci_pci"
#define CFG_DEVICE_DEREGISTER		1

#define CFG_CONSOLE_OVERWRITE_ROUTINE 1

#ifdef CONFIG_440EP
/*Comment this out to enable USB 1.1 device*/
#undef USB_2_0_DEVICE
#endif /*CONFIG_440EP*/

#ifdef DEBUG
#define CONFIG_PANIC_HANG
#else
#define CONFIG_HW_WATCHDOG			/* watchdog */
#endif
				
#define CONFIG_SAMCOMMANDS (CONFIG_CMD_DFL	| \
				CFG_CMD_BDI			| \
				CFG_CMD_DATE		| \
				CFG_CMD_EEPROM		| \
				CFG_CMD_ELF			| \
				CFG_CMD_ENV			| \
				CFG_CMD_EXT2		| \
				CFG_CMD_I2C			| \
				CFG_CMD_MEMORY		| \
				CFG_CMD_MII			| \
				CFG_CMD_NET			| \
				CFG_CMD_NFS			| \
				CFG_CMD_PCI			| \
				CFG_CMD_PING		| \
				CFG_CMD_RUN			| \
				CFG_CMD_USB	)

#define CONFIG_EXCLUDECMDS (CFG_CMD_CONSOLE | \
				CFG_CMD_DOC			| \
				CFG_CMD_FLASH		| \
				CFG_CMD_IRQ			| \
				CFG_CMD_SETGETDCR	| \
				CFG_CMD_IMLS )
										
#define CONFIG_COMMANDS (CONFIG_SAMCOMMANDS & ~CONFIG_EXCLUDECMDS)

#define CONFIG_CMD_BDI
#define CONFIG_CMD_DATE
#define CONFIG_CMD_EEPROM
#define CONFIG_CMD_ELF
#define CONFIG_CMD_ENV
#define CONFIG_CMD_EXT2
#define CONFIG_CMD_I2C
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_MII
#define CONFIG_CMD_NET
#define CONFIG_CMD_NFS
#define CONFIG_CMD_PCI
#define CONFIG_CMD_PING
#define CONFIG_CMD_RUN
#define CONFIG_CMD_USB

/*
 * Miscellaneous configurable options
 */
#define CONFIG_CMD_EDITING	1
#define CFG_LONGHELP					/* undef to save memory		*/
#define CFG_PROMPT	        "] "		/* Monitor Command Prompt	*/
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CBSIZE	        1024		/* Console I/O Buffer Size	*/
#else	
#define CFG_CBSIZE	        256			/* Console I/O Buffer Size	*/
#endif
#define CFG_PBSIZE			(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS			16			/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE /* Boot Argument Buffer Size	*/

#define CFG_MEMTEST_START	0x0400000	 /* memtest works on	        */
#define CFG_MEMTEST_END		0x0C00000 	/* 4 ... 12 MB in DRAM	*/

#define CFG_LOAD_ADDR		0x00100000	/* default load address */
#define CFG_EXTBDINFO		1			/* To use extended board_into (bd_t) */

#define CFG_HZ		        1000		/* decrementer freq: 1 ms ticks */

#define CONFIG_CMDLINE_EDITING	 1		/* add command line history	*/
#define CONFIG_SILENT_CONSOLE    1

/*
 * RTC
 */
#define CONFIG_RTC_M41ST84 		1
#define CFG_I2C_RTC_ADDR 		0x68
#define CFG_M41ST84_BASE_YEAR	2000
 
/*-----------------------------------------------------------------------
 * PCI stuff
 *-----------------------------------------------------------------------
 */
/* General PCI */
#define CONFIG_PCI				/* include pci support	        */
#define CONFIG_PCI_PNP			/* do (not) pci plug-and-play   */
#define CONFIG_PCI_SCAN_SHOW	/* show pci devices on startup  */
#define CFG_PCI_TARGBASE		CFG_PCI_MEMBASE /* PCIaddr mapped to CFG_PCI_MEMBASE*/
#define CFG_PCI_MEMSIZE 		0x40000000
#define CFG_ISA_IO_BASE_ADDRESS	0xe8000000

/* Board-specific PCI */
#define CFG_PCI_PRE_INIT                /* enable board pci_pre_init()  */
#define CFG_PCI_TARGET_INIT
#define CFG_PCI_MASTER_INIT

#define CFG_PCI_SUBSYS_VENDORID 0x10e8	/* AMCC */
#define CFG_PCI_SUBSYS_ID       0xcafe	/* Whatever */

#define CFG_SCSI_SCAN_BUS_REVERSE  		/* Don't touch */

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ		(8 << 20)	/* Initial Memory map for Linux */

/*-----------------------------------------------------------------------
 * External Bus Controller (EBC) Setup
 *----------------------------------------------------------------------*/
#define CFG_FLASH			CFG_FLASH_BASE & 0xfff00000
#define CFG_FPGA			CFG_NVRAM_BASE_ADDR

/* Memory Bank 0 (NOR-FLASH) initialization					*/
#define CFG_EBC_PB0AP		0x03017240 // 0x03017300
#define CFG_EBC_PB0CR		(CFG_FLASH | 0x18000) // 0xda000)

/* Memory Bank 3 (FPGA) initialization						*/
//#define CFG_EBC_PB3AP		0x04814500
//#define CFG_EBC_PB3CR		(CFG_FPGA | 0x5a000) // 0x18000)

/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CFG_DCACHE_SIZE		(32<<10) 	/* For AMCC 440 CPUs			*/
#define CFG_CACHELINE_SIZE	32			/* ...			*/
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CACHELINE_SHIFT	5			/* log base 2 of the above value	*/
#endif

/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD	0x01		/* Normal Power-On: Boot from FLASH	*/
#define BOOTFLAG_WARM	0x02		/* Software reboot			*/

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CONFIG_KGDB_BAUDRATE	230400	/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2		/* which serial port to use */
#endif

/*-----------------------------------------------------------------------
 * ATI FrameBuffer Configuration
 */

/* video */
#undef CONFIG_VIDEO

#if defined(CONFIG_VIDEO)

#define VIDEO_IO_OFFSET		CFG_ISA_IO_BASE_ADDRESS /*PCI video card used*/
#undef CONFIG_BIOSEMU
#define CONFIG_CFB_CONSOLE
#define CONFIG_VIDEO_SW_CURSOR
#define CONFIG_VGA_AS_SINGLE_DEVICE
#define CONFIG_ATI_RADEON_FB
#endif

/*-----------------------------------------------------------------------
 * IDE ATAPI Configuration
 */
#define CONFIG_LBA48			1
#define CONFIG_ATAPI			1
#define CFG_IDE_MAXBUS			2
#define CFG_IDE_MAXDEVICE		4

#define CFG_ATA_BASE_ADDR		0xe8000000
#define CFG_ATA_IDE0_OFFSET		0x1F0
#define CFG_ATA_IDE1_OFFSET		0x170
#define CFG_ATA_REG_OFFSET		0
#define CFG_ATA_DATA_OFFSET		0
#define CFG_ATA_ALT_OFFSET		0x0200

/*
 * Extra ENV stuff
 */
#define CONFIG_EXTRA_ENV_SETTINGS		\
	"stdout=vga\0"						\
	"stdin=usbkbd\0"					\
	"s4sii_maxbus=1\0"					\
	"ide_doreset=on\0"					\
	"ide_reset_timeout=10\0"			\
	"ide_cd_timeout=20\0"				\
	"bootcmd=menu; run menuboot_cmd\0"	\
	"menucmd=menu\0"					\
	"menuboot_cmd=boota\0"				\
	"boot_method=boota\0"				\
	"boot1=s4siicdrom\0"				\
	"boot2=s4sii\0"

// FLASH STUFF
#define CFG_FLASH_WORD_SIZE unsigned char
#define CFG_FLASH_ADDR0 0x555
#define CFG_FLASH_ADDR1 0x2aa

#endif	/* __CONFIG_H */
